#define IS_NUMERIC_KEY(key) ((key >= 0x30) && (key <= 0x39))
#define IS_ALPHA_KEY(key) ((key >= 0x41) && (key <= 0x5A))
#define IS_NUMPAD_KEY(key) ((key >= 0x60) && (key <= 0x6F))
#define IS_OTHER_KEY(key) ((key == VK_SPACE) || (key == VK_BACK) || (key == VK_DELETE))

#define KEY_CHECK(key) (IS_NUMERIC_KEY(key) || IS_ALPHA_KEY(key) || IS_NUMPAD_KEY(key) || IS_OTHER_KEY(key))

#ifdef __BORLANDC__
#define _stricmp stricmp
#endif

#pragma comment (lib, "blackbox.lib")
#define _WIN32_WINNT 0x0500



#include "BBApi.h"
#include <commctrl.h>
#include <shellapi.h>



#include "resource.h"

#include "Broambox.h"



extern "C"
{
  DLL_EXPORT int beginPluginEx (HINSTANCE, HWND);
  DLL_EXPORT int beginPlugin (HINSTANCE);
  DLL_EXPORT void endPlugin (HINSTANCE);
  DLL_EXPORT LPCSTR pluginInfo (int);
};

LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK EditProc (HWND, UINT, WPARAM, LPARAM);

Broambox *pBBox;

int
beginPluginEx (HINSTANCE hInstance, HWND hWnd)
{

  pBBox = new Broambox (hInstance, hWnd);
  return pBBox->CreateBroambox ();
}

int
beginPlugin (HINSTANCE hInstance)
{

  return beginPluginEx (hInstance, NULL);

}

void
endPlugin (HINSTANCE hInstance)
{
  pBBox->SaveSettings();
  pBBox->DestroyBroambox ();
  delete (pBBox);
}



static const char BB_NAME[] = "UberBox";
static const char BB_VERSION[] = "0.1.3 Beta";
static const char BB_AUTHOR[] = "John Williamson / Brian Tresni Hartvigsen";
static const char BB_RELEASE[] = "28 Jan 2005";
static const char BB_LINK[]
= "http://www.dcs.gla.ac.uk/~jhw/uberbox-beta.zip";
static const char BB_EMAIL[] = "johnhw@gmail.com";
static const char BB_DEF[] = "UberBox 0.1.2 Beta John Williamson";
LPCSTR
pluginInfo (int field)
{
  switch (field)
    {
    case PLUGIN_NAME:
      return BB_NAME;
    case PLUGIN_AUTHOR:
      return BB_AUTHOR;
    case PLUGIN_EMAIL:
      return BB_EMAIL;
    case PLUGIN_LINK:
		return BB_LINK;
	case PLUGIN_RELEASE:
	return BB_RELEASE;
    case PLUGIN_VERSION:
      return BB_VERSION;
    case PLUGIN_BROAMS:
      {
        return
          "@BBroamBoxShowAliases"
	  "@BBroamBoxWidth"
	  "@BBroamBoxSaveSettings"
	  "@BBroamBoxReloadSettings"
	  "@BBroamBoxEditSettings"
	  "@BBroamBoxAutohide"
	  "@BBroamBoxAutohideAlways"
	  "@BBroamBoxInSlit"
	  "@BBroamBoxUseRegexs"
	  "@BBroamBoxOverrideBevel" // true false yes no 0 1
	  "@BBroamBoxBevelWidth" //off 1 2 3 ...
	  "@BBroamBoxShowVariables"
	  "@BBroamBoxShowRegexs"
	  "@BBroamBoxRegexDebug"
	  "@BBroamBoxLastStar" // optional 1 2 3 ... for previous
	  "@BBroamBoxStyle"
	  "@BBBroamBoxHide"
          "@BBBroamBoxShow"
          "@BBBroamBoxToggle" "@BBBroamBoxRelay" "@BBBroamBoxFocus";
      }
    default:
      {
		  return BB_DEF;
      }
      break;
    }
}

int is_hidden;
int focusing = 0;
int focused = 0;
#define AUTOHIDE_TIMER 21451

void
autoHide (int hide, HWND win)
{
  RECT winRect;
  RECT drect;
  is_hidden = hide;
  GetWindowRect (win, &winRect);
  GetWindowRect (GetDesktopWindow (), &drect);
  if (hide && pBBox->useAutoHide && !focusing && !focused)
    {
      int width = pBBox->width;
      int height = pBBox->height;
      int x = pBBox->x;
      int y = pBBox->y;
      int ox = x;
      int oy = y;
      if (y <= 0)
        {
          height = 0;

        }
      else if (x <= 0)
        {
          width = 0;
        }
      if (y + height >= drect.bottom)
        {
          height = 0;
          y = drect.bottom;
        }
      else if (x + width >= drect.right)
        {
          width = 0;
          x = drect.right;
        }
      if (pBBox->autoHideEverywhere && height > 1)
        height = 1;

      SetWindowPos (win, NULL, x, y, width, height,
                    SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
	  
	 



      pBBox->x = ox;
      pBBox->y = oy;
    }
  else
    {
      SetWindowPos (win, NULL, pBBox->x, pBBox->y, pBBox->width,
                    pBBox->height, SWP_NOACTIVATE | SWP_NOZORDER);
	  DWORD startSel, endSel;
	  PostMessage(win, EM_SCROLLCARET, NULL, NULL);
	   SendMessage(win,         // handle to destination window 
  EM_GETSEL,           // message to send
  (WPARAM)&startSel,     // starting position (LPDWORD)
  (LPARAM)&endSel);
	SendMessage(win, EM_SETSEL, startSel, endSel);
      pBBox->ReplaceFont ();
    }
  
  if (pBBox->useAutoHide)
    {
      if (focusing)
        {
          focusing--;
          SetTimer (win, AUTOHIDE_TIMER, 1000, NULL);

        }
      else
        {
          SetTimer (win, AUTOHIDE_TIMER, 50, NULL);
        }
    }
}


bool aliasDragDrop = false;
bool inTab = false;
char tabStr[MAX_LINE_LENGTH], curText[MAX_LINE_LENGTH];
int zDelta;
HWND aliasWnd = NULL;
char aliasStr[400];
INT_PTR CALLBACK
AliasDlgFunc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

  HWND textWnd;

  switch (uMsg)
    {

    case WM_INITDIALOG:
      char str[MAX_LINE_LENGTH];
      sprintf (str, "Alias for: %s", curText);
      SetFocus (GetDlgItem (hwndDlg, AliasEdit));
      textWnd = GetDlgItem (hwndDlg, StaticText);
	  if(!aliasDragDrop)
	  {
		ShowWindow(GetDlgItem(hwndDlg, AliasEdit2), SW_HIDE);
		ShowWindow(GetDlgItem(hwndDlg, StaticText2), SW_HIDE);
	  }
	  else
	  {
		ShowWindow(GetDlgItem(hwndDlg, AliasEdit2), SW_SHOW);
		ShowWindow(GetDlgItem(hwndDlg, StaticText2), SW_SHOW);
		SetWindowText(GetDlgItem(hwndDlg, StaticText2), "Arguments");
	  }
      if (textWnd)
        SetWindowText (textWnd, str);
      return true;
      break;

    case WM_CLOSE:

      DestroyWindow (hwndDlg);
      aliasWnd = NULL;
      return TRUE;


    case WM_COMMAND:
      switch (LOWORD (wParam))
        {
          
        case IDCANCEL2:
        case IDCANCEL:
          DestroyWindow (hwndDlg);
          aliasWnd = NULL;

          return TRUE;
          break;

        case IDOK:

          int gotText = GetDlgItemText (hwndDlg,
                                        AliasEdit,
                                        aliasStr,
                                        sizeof (aliasStr));

          if (gotText)
            {
			  char args[MAX_LINE_LENGTH];
			  if(aliasDragDrop && GetDlgItemText(hwndDlg, AliasEdit2, args, sizeof(args)))
			  {
				if(strlen(args)>0)
				strcat(curText, " ");
				strcat(curText, args);
			  }

              pBBox->AddAlias (aliasStr, curText);

              char tmp[MAX_LINE_LENGTH];
              sprintf (tmp, "%s %s\n", aliasStr, curText);
              pBBox->WriteAlias (tmp);


            }
          DestroyWindow (hwndDlg);
          aliasWnd = NULL;

          return TRUE;
          break;
        }
    default:
      return false;
    }



}


void
AddAlias (char *str, HWND hWnd, bool dragdrop)
{
  if (strlen (str) > 1)
    {
      strcpy (curText, str);
      RECT rect;
      RECT drect;
      GetWindowRect (GetDesktopWindow (), &drect);
      GetWindowRect (hWnd, &rect);
      int bx = rect.left + 100;
      int by = rect.top + pBBox->height + 2;
      int dx = 0, dy = 0;
      if (bx > drect.right)
        dx = drect.right - bx;
      if (by > drect.bottom)
        dy = drect.bottom - by;

	  aliasDragDrop = dragdrop;

      DialogBox (pBBox->hMainInst, MAKEINTRESOURCE (AliasDlg), NULL,
                 AliasDlgFunc);

      ShowWindow (aliasWnd, SW_SHOW);

      return;
    }

}

int ParseBool(char *lwr)
{

	if(lwr && (strstr(lwr, "yes")==lwr || strstr(lwr, "on")==lwr || strstr(lwr, "true")==lwr || strstr(lwr, "1")==lwr))
		return 1;
	else
		return 0;

}

POINT PrevPoint;
bool prevValid = false;
LRESULT CALLBACK
EditProc (HWND hText, UINT msg, WPARAM wParam, LPARAM lParam)
{

HDROP hp;

  switch (msg)
    {
    case WM_TIMER:
      {
        if (wParam == AUTOHIDE_TIMER)
          {
            POINT pt;
            RECT rect;
            GetCursorPos (&pt);
            GetWindowRect (hText, &rect);

            if (pt.x >= pBBox->x && pt.x <= pBBox->x + pBBox->width
                && pt.y >= pBBox->y && pt.y <= pBBox->y + pBBox->height)
              {

                autoHide (0, hText);
              }
            else
              autoHide (1, hText);
          }

      }
      break;
    case WM_SETFOCUS:
      {
		focused = 1;
        autoHide (0, hText);
        SendMessage (hText, EM_SETSEL, 0, -1);
        pBBox->ToggleSlitAutoHide (true);
        KillTimer (hText, AUTOHIDE_TIMER);
        SetTimer (hText, AUTOHIDE_TIMER, 1000, NULL);
		
      }
      break;

    case WM_KILLFOCUS:
      {
	    focused = 0;
        autoHide (1, hText);
		
        pBBox->ToggleSlitAutoHide (false);
		SetWindowPos (hText, HWND_NOTOPMOST, 0, 0, 0, 0,
                          SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);


      }
      break;

    case WM_MOUSEMOVE:
      {

        autoHide (0, hText);


      }
    case WM_NCHITTEST:
      {
        if ((GetAsyncKeyState (VK_CONTROL) & 0x8000))
          return HTCAPTION;
      }
      break;

    case WM_NCRBUTTONUP:
      {

        autoHide (0, hText);
        return pBBox->RButtonDown (hText, msg, wParam, lParam);
      }

    case WM_MOVE:
      pBBox->x = (int) (short) LOWORD (lParam);
      pBBox->y = (int) (short) HIWORD (lParam);
      pBBox->WritePosition();
      break;
    case WM_MOUSEWHEEL:
      zDelta = GET_WHEEL_DELTA_WPARAM (wParam);
      if (zDelta < 0)
        wParam = VK_DOWN;
      else if (zDelta > 0)
        wParam = VK_UP;
    case WM_KEYDOWN:
      {
	    if(wParam == VK_ESCAPE)
		{
		
			if(prevValid)
			{
			POINT pt;
			GetCursorPos (&pt);
	    		SetCursorPos (PrevPoint.x, PrevPoint.y);
		       	mouse_event (MOUSEEVENTF_MIDDLEDOWN, PrevPoint.x, PrevPoint.y, 0, NULL);
		Sleep(20);

            mouse_event (MOUSEEVENTF_MIDDLEUP, PrevPoint.x, PrevPoint.y, 0, NULL);
				mouse_event (MOUSEEVENTF_MIDDLEDOWN, PrevPoint.x, PrevPoint.y, 0, NULL);
     

            mouse_event (MOUSEEVENTF_MIDDLEUP, PrevPoint.x, PrevPoint.y, 0, NULL);
			
            SetCursorPos (pt.x, pt.y);
			}


		}
        if ((GetAsyncKeyState (VK_CONTROL) & 0x8000) && wParam == 0x4C) // Ctrl-L key
          {
            char tmp[MAX_LINE_LENGTH];
            GetWindowText (hText, tmp, sizeof (tmp) - 1);
            AddAlias (tmp, hText, false);
            return false;

          }
        if ((GetAsyncKeyState (VK_CONTROL) & 0x8000) && wParam == 0x49) //Ctrl-I

          {
            char txt[MAX_LINE_LENGTH];
            char broadStr[MAX_LINE_LENGTH];
            GetWindowText (hText, txt, sizeof (txt));
            sprintf (broadStr, "@BBInterface Control Create Button %s", txt);


            PostMessage (pBBox->hBB4Win, BB_BROADCAST, 0,
                         (LPARAM) strdup (broadStr));
            sprintf (broadStr,
                     "@BBInterface Control SetAgent %s MouseUp Bro@m \"@BBBroamBoxRelay %s\"",
                     txt, txt);
            MessageBox (NULL, broadStr, NULL, MB_OK);
            PostMessage (pBBox->hBB4Win, BB_BROADCAST, 0,
                         (LPARAM) strdup (broadStr));
            sprintf (broadStr,
                     "@BBInterface Control SetAgent %s Caption StaticText \"%s\"",
                     txt, txt);

            PostMessage (pBBox->hBB4Win, BB_BROADCAST, 0,
                         (LPARAM) strdup (broadStr));


          }



        autoHide (0, hText);
        KillTimer (hText, AUTOHIDE_TIMER);
        SetTimer (hText, AUTOHIDE_TIMER, 1000, NULL);
        if (wParam == VK_TAB)
          {
            char cur_text[255];
            GetWindowText (hText, cur_text, 255);

            if (GetAsyncKeyState (VK_SHIFT) & 0x8000)
              {
                pBBox->AutoCompletePath (cur_text);
              }
            else
              {
                if (!inTab)
                  {
                    strcpy (tabStr, cur_text);
                    pBBox->AutoCompleteNext (cur_text, 1);
                    inTab = true;
                  }
                else
                  {
                    pBBox->AutoCompleteNext (tabStr, 1);
                  }
                return 0;
              }
          }

        if (wParam == VK_DOWN)
          {
            char cur_text[255];
            GetWindowText (hText, cur_text, 255);
            if (strlen (cur_text) < 1 || (inTab && strlen (tabStr) < 1))
              {
                pBBox->HistoryPrev ();
                inTab = 1;
                strcpy (tabStr, "");
              }
            else
              {
                if (!inTab)
                  {
                    strcpy (tabStr, cur_text);
                    pBBox->AutoCompleteNext (cur_text, 0);
                    inTab = true;
                  }
                else
                  {
                    pBBox->AutoCompleteNext (tabStr, 0);
                  }

              }
            return 0;
          }
        if (wParam == VK_UP)
          {
            char cur_text[255];
            GetWindowText (hText, cur_text, 255);
            if (strlen (cur_text) < 1 || (inTab && strlen (tabStr) < 1))
              {
                pBBox->HistoryNext ();
                inTab = 1;
                strcpy (tabStr, "");
              }
            else
              {
                if (!inTab)
                  {
                    strcpy (tabStr, cur_text);
                    pBBox->AutoCompletePrev (cur_text);
                    inTab = true;
                  }
                else
                  {
                    pBBox->AutoCompletePrev (tabStr);
                  }
              }
            return 0;
          }
        if (KEY_CHECK (wParam))
          InvalidateRect (hText, NULL, true);
      }
      break;

    case WM_CHAR:
      {
	    focused = 1;
        if (wParam == VK_TAB || wParam == VK_UP || wParam == VK_DOWN)
          {
            return 0;
          }
        if (pBBox->LastACHandle)
          FindClose (pBBox->LastACHandle);
        pBBox->LastACHandle = NULL;
        pBBox->AutoCompleteReset ();
        inTab = false;

        if (wParam == VK_RETURN)
          {
            pBBox->Execute ();
            pBBox->ToggleSlitAutoHide (false);
            return 0;
          }
      }
      break;

    case WM_CLOSE:
      {
        return 0;
      }
      break;                    // no ALT+F4

      //gr
    case WM_PAINT:
      {
        PAINTSTRUCT ps;

        HDC hdc = BeginPaint (hText, &ps);
        RECT r;
        GetClientRect (hText, &r);

        HDC buf = CreateCompatibleDC (NULL);
        HGDIOBJ oldbuf =
          SelectObject (buf, CreateCompatibleBitmap (hdc, r.right, r.bottom));

        pBBox->PaintBroambox (buf, &r);

        CallWindowProc (pBBox->wpEditProc, hText, msg, (WPARAM) buf, lParam);

        BitBlt (hdc, 0, 0, r.right, r.bottom, buf, 0, 0, SRCCOPY);

        DeleteObject (SelectObject (buf, oldbuf));
        DeleteDC (buf);

        EndPaint (hText, &ps);
        return 0;
      }
      break;

    case WM_CTLCOLOREDIT:
      {
        //gr
        SetTextColor ((HDC) wParam, pBBox->textColor);
        SetBkMode ((HDC) wParam, TRANSPARENT);
        return (LRESULT) GetStockObject (NULL_BRUSH);
      }
      break;

    case WM_ERASEBKGND:
      return TRUE;
	case WM_DROPFILES:
		char fileBuf[MAX_LINE_LENGTH];
		hp = (HDROP) wParam;
		DragQueryFile(hp,0,fileBuf,sizeof(fileBuf));
		AddAlias (fileBuf, hText, true);
		DragFinish(hp);
		return 0;
		break;

    case BB_BROADCAST:
      {
        char temp[MAX_LINE_LENGTH];
        strcpy (temp, (LPCSTR) lParam);
        if (!_stricmp (temp, "@BBBroamBoxFocus"))
          {
            focused = 1;
			RECT rect;
            POINT pt;
			prevValid = true;
            EnableWindow (hText, true);
            GetWindowRect (hText, &rect);
            ShowWindow (hText, SW_SHOWNORMAL);
            SetFocus (hText);
            SetWindowPos (hText, HWND_TOP, 0, 0, 0, 0,
                          SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
            autoHide (0, hText);

            GetCursorPos (&pt);
            SetCursorPos (rect.left, rect.top);
            mouse_event (MOUSEEVENTF_LEFTDOWN, rect.left, rect.top, 0, NULL);
            mouse_event (MOUSEEVENTF_MOVE, rect.left, rect.top, 0, NULL);
			SetWindowPos (hText, HWND_TOPMOST, 0, 0, 0, 0,
		                         SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);

            

            mouse_event (MOUSEEVENTF_LEFTUP, rect.left, rect.top, 0, NULL);
			
            SetCursorPos (pt.x, pt.y);
			PrevPoint.x = pt.x;
			PrevPoint.y = pt.y;
          }
        char lwr[MAX_LINE_LENGTH];
        strcpy (lwr, temp);
        _strlwr (lwr);
	
        char *args = strchr (temp, ' ');
		if(args)
			args++;


        if (strstr (lwr, "@bbbroamboxrelay") == lwr)
          {
            char tmp[MAX_LINE_LENGTH];
            GetWindowText (hText, tmp, sizeof (tmp));
            if (args)
              SetWindowText (hText, args );
            pBBox->Execute ();
            SetWindowText (hText, tmp);
          }

	if(strstr(lwr, "@bbbroamboxstyle")==lwr)
	{
	  pBBox->SaveSettings();
	  if(args)
		strcpy(pBBox->configStyle,args);
	  else
		  strcpy(pBBox->configStyle, "");
	  pBBox->ReadSettings();
	}

	if(strstr(lwr, "@bbbroamboxshowaliases")==lwr)
	{
		pBBox->ShowAliases();
	}
	
	if(strstr(lwr, "@bbbroamboxshowregexs")==lwr)
	{
		pBBox->ShowRegexs();
	}

	
	if(strstr(lwr, "@bbbroamboxshowvariables")==lwr)
	{
		pBBox->ShowVariables();
	}

	if(strstr(lwr, "@bbbroamboxwidth")==lwr)
	{
		if(args)
		{
			pBBox->width = (int)strtol(args,NULL,10);
        	pBBox->SizeBroambox();
		}
	}
	if(strstr(lwr, "@bbbroamboxsavesettings")==lwr)
	{
		pBBox->SaveSettings(); 
	}
	if(strstr(lwr, "@bbbroamboxreloadsettings")==lwr)
	{
		pBBox->ReadSettings();
		pBBox->SizeBroambox();
		pBBox->ReplaceFont();
		
	}
	if(strstr(lwr, "@bbbroamboxuseregexs")==lwr)
	{
		pBBox->use_regexs = !pBBox->use_regexs;
	}
	if(strstr(lwr, "@bbbroamboxregexdebug")==lwr)
	{
		pBBox->regex_debug = !pBBox->regex_debug;
	}
	if(strstr(lwr, "@bbbroamboxautohidealways")==lwr)
	{
		pBBox->autoHideEverywhere = !pBBox->autoHideEverywhere;
	}
	else if(strstr(lwr, "@bbbroamboxautohide")==lwr)
	{
		pBBox->useAutoHide = !pBBox->useAutoHide;
	}
	
	if(strstr(lwr, "@bbbroamboxinslit")==lwr)
	{
		bool inslit = !pBBox->inSlit;
		if(inslit)
			pBBox->EnterSlit();
		else
		    pBBox->LeaveSlit();
	}


	if(strstr(lwr, "@bbbroamboxlaststar")==lwr)
	{
           char tempedit[MAX_LINE_LENGTH], tempfile[MAX_LINE_LENGTH];
	   tempfile[0] = 0;
 	   pBBox->GetVarValue("tmpstar", tempfile);
	   if(strlen(tempfile)>1)
	   {
            sprintf(tempedit, "@BBCore.edit %s", tempfile);
	        PostMessage (pBBox->hBB4Win, BB_BROADCAST, 0, (LPARAM) strdup (tempedit));	    
	   }

	}
	
        if(strstr(lwr, "@bbbroamboxoverridebevel")==lwr)
	{
	
                pBBox->SetBevel(args);
		pBBox->SizeBroambox();
	}
	
       if(strstr(lwr, "@bbbroamboxbevelwidth")==lwr)
	{
		if(args)
		{
			pBBox->bevelWidth = (int)strtol(args, NULL, 10);
			pBBox->SizeBroambox();
		}
	}
       if(strstr(lwr, "@bbbroamboxeditsettings")==lwr)
	{
	    char tempedit[MAX_LINE_LENGTH];
	   sprintf (tempedit, "@BBCore.edit %s\\%s", pBBox->localDir, "broambox.rc");
	    PostMessage (pBBox->hBB4Win, BB_BROADCAST, 0, (LPARAM) strdup (tempedit));	    
	}

	
       
        if (!_stricmp (temp, "@BBBroamBoxToggle"))
          {
            if (!pBBox->shown)
              pBBox->ShowWindow ();
            else
              pBBox->HideWindow ();
          }
        if (!_stricmp (temp, "@BBBroamBoxShow"))
          {
            pBBox->ShowWindow ();

          }
        if (!_stricmp (temp, "@BBBroamBoxHide"))
          {
            pBBox->HideWindow ();
          }

        if (!_stricmp (temp, "@BBShowPlugins"))
          {
            pBBox->ShowWindow ();
          }
        else if (!_stricmp (temp, "@BBHidePlugins"))
          {
            pBBox->HideWindow ();
          }
        if (IsInString (temp, "@Broambox"))
          {
            char broam[MAX_LINE_LENGTH], command[MAX_LINE_LENGTH],
              arguments[MAX_LINE_LENGTH];
            char *tokens[2];
            tokens[0] = broam;
            tokens[1] = command;

            BBTokenize (temp, tokens, 2, arguments);

            if (!_stricmp (command, "Toggle"))
              {
                if (!_stricmp (arguments, "Slit"))
                  pBBox->ToggleInSlit ();
              }
          }
      }
      return 0;


  case BB_RECONFIGURE:
      {
        
        pBBox->ReadSettings ();
             
        pBBox->SizeBroambox ();
        pBBox->ReplaceFont ();



      } 
      return 0;
    }
  return CallWindowProc (pBBox->wpEditProc, hText, msg, wParam, lParam);
}
