#define _CRTDBG_MAP_ALLOC

#define GET_X_LPARAM(lParam)   ((int)(short)LOWORD(lParam))
#define GET_Y_LPARAM(lParam)   ((int)(short)HIWORD(lParam))
#define DBGR
#define _stricmp stricmp

#define _WIN32_WINNT 0x0500
#define _WIN32_IE 0x0500


#pragma comment (lib, "blackbox.lib")

#include <cstdlib>
#include <crtdbg.h>
#include <ctime>
#include <windows.h>
#include <winuser.h>
#include <psapi.h>
#include <shellapi.h>
#include "BBApi.h"
#include <mmsystem.h>

#include "pcre.h"
#include "pcrs.h"

#include "Broambox.h"


#pragma comment (lib, "pcre.lib")
#pragma comment (lib, "psapi.lib")
#pragma comment (lib, "user32.lib")

extern LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM);
extern LRESULT CALLBACK EditProc (HWND, UINT, WPARAM, LPARAM);
int messages[] = { BB_BROADCAST, BB_RECONFIGURE, 0 };


Broambox::Broambox (HINSTANCE hInstance, HWND hWnd)
{
  hMainInst = hInstance;

  hBB4Win = GetBBWnd ();
  hSlit = hWnd;
  hEditWnd = NULL;

  hEditFont = NULL;

  mainStyle = NULL;
  editStyle = NULL;
  overStyle = NULL;
  mainMenu = NULL;

  inSlit = false;

  tempSlit = false;
  noWho = false;
  no8Ball = false;

  historyIndex = 0;
  historyCount = 0;
  ZeroMemory (&history, sizeof (history));


  _CrtSetDbgFlag (_CRTDBG_CHECK_CRT_DF | _CRTDBG_ALLOC_MEM_DF |
                  _CRTDBG_LEAK_CHECK_DF | _CRTDBG_CHECK_ALWAYS_DF);
  _CrtSetReportMode (_CRT_ERROR, _CRTDBG_MODE_DEBUG);


  ReadSettings ();
}

bool Broambox::CreateBroambox ()
{


  hEditWnd = CreateWindowEx (WS_EX_TOOLWINDOW | WS_EX_ACCEPTFILES,
                             "EDIT",
                             NULL,
                             WS_POPUP | WS_VISIBLE | ES_AUTOHSCROLL |
                             ES_MULTILINE, x, y, width, height, NULL, NULL,
                             hMainInst, NULL);


  strcpy(configStyle, "");
  strcpy(failSound, "");
  strcpy(successSound, "");

  wpEditProc =
    (WNDPROC) SetWindowLong (hEditWnd, GWL_WNDPROC, (long) EditProc);

  SendMessage (hBB4Win, BB_REGISTERMESSAGE, (WPARAM) hEditWnd,
               (LPARAM) messages);
  MakeSticky (hEditWnd);

  if (InSlit ())
    SendMessage (hSlit, SLIT_ADD, NULL, (LPARAM) hEditWnd);

  ReplaceFont ();
return 0;

}

void
Broambox::DestroyBroambox ()
{

  if (mainStyle)
    delete mainStyle;
  if (editStyle)
    delete editStyle;
  if (overStyle)
	  delete overStyle;

 
  SendMessage (hBB4Win, BB_UNREGISTERMESSAGE, (WPARAM) hEditWnd,
               (LPARAM) messages);
  RemoveSticky (hEditWnd);

  if (hEditFont)
    DeleteObject (hEditFont);
  DestroyWindow (hEditWnd);

  UnregisterClass (GetAppName (), hMainInst);
}

void
Broambox::MoveBroambox (WINDOWPOS * wndPos)
{
  x = wndPos->x;
  y = wndPos->y;

WritePosition();
}

void
Broambox::SizeBroambox ()
{
	
  if (ReadBool
      (extensionsrcPath (), "blackbox.appearance.bullet.unix:", true))
    height = fontHeight + 2 + (2 * bevel) + (border * 4);
  else
    height = fontHeight + 2 + (border * 2);
  if (borderWidth != -1)
    height = fontHeight + 2 + borderWidth * 2;
  if (bevelWidth != -1)
    height += bevelWidth * 2 + 2;

  MoveWindow (hEditWnd, x, y, width, height, TRUE);
}


void
Broambox::PaintBroambox (HDC hdc, LPRECT rp)
{

  RECT bounds = *rp;
  
  int bevelStyle = mainStyle->bevelstyle;
  int editBevelStyle = editStyle->bevelstyle;

  if (borderStyle & BORDER_BEVEL_RAISED)
    {
      bevelStyle = BEVEL_RAISED;
      editBevelStyle = BEVEL_RAISED;
    }
  if (borderStyle & BORDER_BEVEL_SUNKEN)
    {
      bevelStyle = BEVEL_SUNKEN;
      editBevelStyle = BEVEL_SUNKEN;
    }
  if (borderStyle & BORDER_NO_BEVEL)
    {
      bevelStyle = BEVEL_FLAT;
      editBevelStyle = BEVEL_FLAT;
    }
  if (borderStyle & BORDER_EDGE)
    border = 1;
  if (borderWidth != -1)
    border = borderWidth;
  if (bevelWidth != -1)
    bevel = bevelWidth;

  if (borderStyle & BORDER_NO_EDGE)
    border = 0;


  if (overStyle)
    {

      MakeGradient (hdc, *rp, mainStyle->type, mainBackColor, mainBackColorTo,
                    mainStyle->interlaced, overStyle->bevelstyle,
                    overStyle->bevelposition, bevel, borderColor, 0);

    }
  else
 
    MakeGradient (hdc, *rp, mainStyle->type, mainBackColor, mainBackColorTo,
                  mainStyle->interlaced, bevelStyle,
                  mainStyle->bevelposition, bevel, borderColor, 0);
				  
	    

  if (!editStyle->parentRelative
      || (overStyle && (!overStyle->parentRelative)))
    {
      bounds.bottom -= bevel + border;
      bounds.top += bevel + border;
      bounds.right -= bevel + border;
      bounds.left += bevel + border;
      if (overStyle)
        {
          MakeGradient (hdc, bounds, editStyle->type, editBackColor,
                        editBackColorTo, editStyle->interlaced,
                        overStyle->bevelstyle, overStyle->bevelposition,
                        bevel, borderColor, 0);

        }
      else
        {
          MakeGradient (hdc, bounds, editStyle->type, editBackColor,
                        editBackColorTo, editStyle->interlaced,
                        editBevelStyle, editStyle->bevelposition,
                        bevel, borderColor, 0);
        }


    }

}


void
Broambox::reverseHistory ()
{
  int i, j;
  if (!historyCount)
    return;

  char tmp[MAX_LINE_LENGTH];
  i = 0;
  j = historyCount - 1;

  while (i < j)
    {
      strcpy (tmp, history[i]);
      strcpy (history[i], history[j]);
      strcpy (history[j], tmp);
      i++;
      j--;
    }
}

void
Broambox::FindRegexs (char *regex)
{
  int erroffset;
  char *curPos;
  char *curEnd;
  char tmp[MAX_LINE_LENGTH];
  pcrs_job *job=NULL;
  int escaped = 0;
  curEnd = regex;
  curPos = regex;

  if (*curEnd == 's' && *(curEnd + 1) == '/')
    {
      curEnd += 2;              //skip /s
      while ((*curEnd != '/' || escaped) && *curEnd != '\0')
        {
          escaped = 0;
          if (*curEnd == '\\')
            escaped = 1;
          curEnd++;
        }
      curEnd++;
      while ((*curEnd != '/' || escaped) && *curEnd != '\0')
        {
          escaped = 0;
          if (*curEnd == '\\')
            escaped = 1;
          curEnd++;
        }
      curEnd++;
      while ((*curEnd == 'g' || *curEnd == 'i' || *curEnd == 'm'
              || *curEnd == 'x' || *curEnd == 's') && *curEnd != '\0')
        curEnd++;

      while ((*curEnd == ' ') && *curEnd != '\0')
        curEnd++;
      strcpy (tmp, curPos);
      tmp[(curEnd - curPos)] = '\0';

      regexs[n_regex] = pcrs_compile_command (tmp, &erroffset);
      job = regexs[n_regex];

      if (regexs[n_regex] == NULL)
        MessageBox (NULL, pcrs_strerror (erroffset), NULL, MB_OK);

    }
  curPos = curEnd;
  if(job)
  {
  while (*curEnd == 's' && *(curEnd + 1) == '/')
    {

      curEnd += 2;              //skip /s
      while ((*curEnd != '/' || escaped) && *curEnd != '\0')
        {
          escaped = 0;
          if (*curEnd == '\\')
            escaped = 1;
          curEnd++;
        }
      curEnd++;
      while ((*curEnd != '/' || escaped) && *curEnd != '\0')
        {
          escaped = 0;
          if (*curEnd == '\\')
            escaped = 1;
          curEnd++;
        }
      curEnd++;
      while ((*curEnd == 'g' || *curEnd == 'i' || *curEnd == 'm'
              || *curEnd == 'x' || *curEnd == 's') && *curEnd != '\0')
        curEnd++;
      while ((*curEnd == ' ') && *curEnd != '\0')
        curEnd++;

      strcpy (tmp, curPos);
      tmp[(curEnd - curPos)] = '\0';

      job->next = pcrs_compile_command (tmp, &erroffset);
      job = job->next;
      if (job == NULL)
        MessageBox (NULL, pcrs_strerror (erroffset), NULL, MB_OK);
      curPos = curEnd;
    }
  }
  n_regex++;
}

void
Broambox::AddVariable (char *var, char *val)
{
  if (VarTrie == NULL)
    VarTrie = new trie;
  VarTrie->AddString (var, val);

}


int
Broambox::ParseVariable (char *var_string)
{
  int ovector[30];
  int substr;
  char *var;
  char *val;
  substr =
    pcre_exec (var_match, var_extra, var_string, strlen (var_string), 0, 0,
               ovector, 90);



  if (substr == 3)
    {
      pcre_get_substring (var_string, ovector, substr, 1,
                          (const char **) &var);
      pcre_get_substring (var_string, ovector, substr, 2,
                          (const char **) &val);
      AddVariable (var, val);


      return 1;
    }

  return 0;

}

void
Broambox::ParseEnvironmentVariables ()
{
  char *envStrs = GetEnvironmentStrings ();
  char *ev = envStrs;
  char *laststart = envStrs;
  while (*ev || *(ev + 1))
    {
      if (!*ev)
        {

          ParseVariable (laststart);
          *ev = '$';
          laststart = ev;

        }
      ev++;
    }


}




void
Broambox::AddAlias (char *alias, char *expands)
{
  if (AliasTrie == NULL)
    AliasTrie = new trie;
  if (expands)
    AliasTrie->AddString (alias, expands);
  else
    AliasTrie->AddString (alias, "");

}

void
Broambox::SetupVariables (void)
{
  char path[MAX_LINE_LENGTH];
    const char *error;
	int nLen;
  int erroff;
  AliasTrie = NULL;
  VarTrie = NULL;
  LastACHandle = NULL;
  RegexList = NULL;
  var_match =
    pcre_compile ("\\$([a-zA-Z0-9_]*)\\s*=\\s*(.*)", 0, &error, &erroff,
                  NULL);
  if (!var_match)
    MessageBox (NULL, error, NULL, MB_OK);

  var_extra = pcre_study (var_match, 0, &error);
  urlre = pcre_compile("^\\s*[a-zA-Z_\\.\\/~#%?\"&]*\\.(com|[a-z][a-z]|net|org|gov|mil|biz|edu)(:[0-9]+)?(\\/[a-zA-Z_\\.\\/~#%?\"&]*)?", 0, &error, &erroff, NULL);
urlre_extra = pcre_study(urlre, 0, &error);
  /* Read the enivronment block */
  ParseEnvironmentVariables ();
  /* Get the module path */
  GetModuleFileName (hMainInst, path, sizeof (path));
  nLen = strlen (path) - 1;
  while (nLen > 0 && path[nLen] != '\\')
    nLen--;
  path[nLen + 1] = 0;

  strcpy (localDir, path);

  lastTabCompleteIndex = 0;
}

void
Broambox::ReadHistory (void)
{

  char pathh[MAX_LINE_LENGTH];

  sprintf (pathh, "%shistory.rc", localDir);

  historyCount = 0;

  // Load History 
  FILE *f_hist = fopen (pathh, "r");
  if (!f_hist)
    {
      sprintf (pathh, ".\\history.rc");
      f_hist = fopen (pathh, "r");
    }


  if (f_hist)
    {
      char histbuf[MAX_LINE_LENGTH];
      char *hist_read;
      do
        {
          hist_read = fgets (histbuf, sizeof (histbuf), f_hist);
          if (hist_read)
            {
              hist_read[strlen (hist_read) - 1] = '\0';
              strcpy (history[historyCount++], hist_read);
            }
        }
      while (hist_read);

      fclose (f_hist);
    }
  reverseHistory ();

}

void
Broambox::ReadAliases (void)
{
  char patha[MAX_LINE_LENGTH];

  sprintf (patha, "%saliases.rc", localDir);
  n_regex = 0;
  FILE *f_ali = fopen (patha, "r");
  if (!f_ali)
    {
      sprintf (patha, ".\\aliases.rc");
      f_ali = fopen (patha, "r");
    }

  if (f_ali)
    {
      char alibuf[MAX_LINE_LENGTH],  *alival;
      char *ali_read;
      do
        {
          ali_read = fgets (alibuf, sizeof (alibuf), f_ali);
          if (ali_read)
            {
              if (*ali_read == '$')
                ParseVariable (ali_read);
              else if (strlen (ali_read) > 2 && *ali_read == 's'
                       && *(ali_read + 1) == '/')
	      {
      		if(RegexList==NULL)
			RegexList = new StrList(strdup(ali_read));
		else
			RegexList->AddElement(strdup(ali_read));

                FindRegexs (ali_read);
	      }
              else
                {
                  alival = strchr (ali_read, ' ');

                  if (alival)
                    {
                      *(alival++) = '\0';
                      //Chop newlines
                      alival[strlen (alival) - 1] = '\0';
                    }
                  AddAlias (ali_read, alival);

                }
            }
        }
      while (ali_read);

      fclose (f_ali);
    }

}

void Broambox::SetBevel(char *temp)
{
	borderStyle = 0;
  if (strstr (temp, "raised"))
    borderStyle |= BORDER_BEVEL_RAISED;
  if (strstr (temp, "sunken"))
    borderStyle |= BORDER_BEVEL_SUNKEN;
  if (strstr (temp, "edge"))
    borderStyle |= BORDER_EDGE;
  if (strstr (temp, "noborder"))
    borderStyle |= BORDER_NO_EDGE;
  if (strstr (temp, "nobevel"))
    borderStyle |= BORDER_NO_BEVEL;

}

void Broambox::WritePosition(void)
{
char path[MAX_LINE_LENGTH];
sprintf(path, "%s\\%s", localDir, "broambox.rc");

 WriteInt (path, "broambox.x:", x);
 WriteInt (path, "broambox.y:",y);

}

void Broambox::SaveSettings(void)
{
char path[MAX_LINE_LENGTH];
sprintf(path, "%s\\%s", localDir, "broambox.rc");

 WriteInt (path, "broambox.x:", x);
 WriteInt (path, "broambox.y:",y);
 WriteString(path, "broambox.failwav:", failSound);
 WriteString(path, "broambox.successwav:", successSound);
  
  
 

  WriteInt (path, "broambox.borderWidth:", borderWidth);
  WriteInt (path, "broambox.bevelWidth:", bevelWidth);
  


  WriteInt (path, "broambox.width:", width);
  
  
  WriteBool (path, "broambox.inslit:", inSlit);
  WriteBool (path, "broambox.autohide:", useAutoHide);
  WriteBool (path, "broambox.autohidealways:", autoHideEverywhere);
  WriteBool (path, "broambox.regexdebug:", regex_debug);
  WriteBool (path, "broambox.regex:", use_regexs);

}

void
Broambox::ReadSettings ()
{

  char path[MAX_LINE_LENGTH];
  char path2[MAX_LINE_LENGTH];
  char path3[MAX_LINE_LENGTH];
  
 

  SetupVariables ();
  ReadHistory ();
  ReadAliases ();


  strcpy (path, localDir);

  strcpy (path2, path);
  strcpy (path3, path);
  strcat (path2, "broambox.rc");
  strcat (path3, "broamboxrc");

  if (FileExists (path3))
    strcpy (path, path3);
  else
    strcpy (path, path2);

  x = ReadInt (path, "broambox.x:", 0);
  y = ReadInt (path, "broambox.y:", 0);

  char temp[MAX_LINE_LENGTH], temp2[MAX_LINE_LENGTH];
  strcpy (temp, ReadString (path, "broambox.borderStyle:", "border"));

  _strlwr(temp);
  SetBevel(temp);
  borderWidth = ReadInt (path, "broambox.borderWidth:", -1);
  bevelWidth = ReadInt (path, "broambox.bevelWidth:", -1);
  char bevelStyle[MAX_LINE_LENGTH];
  strcpy (bevelStyle, ReadString (path, "broambox.bevelStyle:", ""));

  width = ReadInt (path, "broambox.width:", 300);
  transparent = ReadBool (path, "broambox.transparent:", true);
  alpha = ReadInt (path, "broambox.alpha:", 100);
  strcpy(failSound, ReadString(path, "broambox.failwav:", ""));
  strcpy(successSound, ReadString(path, "broambox.successwav:", ""));
  inSlit = ReadBool (path, "broambox.inslit:", false);
  useAutoHide = ReadBool (path, "broambox.autohide:", true);
  autoHideEverywhere = ReadBool (path, "broambox.autohidealways:", false);
  regex_debug = ReadBool (path, "broambox.regexdebug:", false);
  use_regexs = ReadBool (path, "broambox.regex:", false);

  noWho = ReadBool (path, "broambox.easteregg.noWho:", false);
  no8Ball = ReadBool (path, "broambox.easteregg.no8Ball:", false);
  shown = true;
  // Get the path to the current style file from Blackbox...

  char style[MAX_LINE_LENGTH], styleStr[MAX_LINE_LENGTH];
  if(strlen(configStyle)<2)
	strcpy (style, ReadString (path, "broambox.style:", "toolbar"));
  else
    strcpy(style, configStyle);
  strcpy(configStyle, style);
  strcpy (path, stylePath ());

  // Get the applicable color settings from the current style...
  sprintf (styleStr, "%s.color:", style);

  mainBackColor = ReadColor (path, styleStr, "#000000");
  sprintf (styleStr, "%s.colorTo:", style);
  mainBackColorTo = ReadColor (path, styleStr, "#FFFFFF");
  sprintf (styleStr, "%s.textColor:", style);
  strcpy (path2, ReadString (path, styleStr, "#000000"));
  sprintf (styleStr, "%s.label.color:", style);
  editBackColor = ReadColor (path, styleStr, "#000000");
  sprintf (styleStr, "%s.label.colorTo:", style);
  editBackColorTo = ReadColor (path, styleStr, "#FFFFFF");

  sprintf (styleStr, "%s.label.textColor:", style);
  textColor = ReadColor (path, styleStr, path2);

  borderColor = ReadColor (path, "borderColor:", "#000000");


  strcpy (temp2, ReadString (path, "*font:", "courier new"));
  strcpy (temp, ReadString (path, "toolbar.font:", temp));
  sprintf (styleStr, "%s.font:", style);
  strcpy (fontFace, ReadString (path, styleStr, temp));
  sprintf (styleStr, "%s.fontHeight:", style);
  fontHeight = ReadInt (path, styleStr, 12);

  // ...gradient type, bevel etc. (using a StyleItem)...
  char tempstyle[MAX_LINE_LENGTH];

  sprintf (styleStr, "%s:", style);
  strcpy (tempstyle, ReadString (path, styleStr, "Flat Gradient Vertical"));
  if (overStyle)
    delete overStyle;

  overStyle = NULL;
  if (strlen (bevelStyle) > 1)
    {
      char tmp[MAX_LINE_LENGTH];
      sprintf (styleStr, "%s:", bevelStyle);
      overStyle = new StyleItem;
      strcpy (tmp, ReadString (path, styleStr, "Flat Gradient Vertical"));
      ParseItem (tmp, overStyle);

    }


  if (mainStyle)
    delete mainStyle;
  mainStyle = new StyleItem;
  ParseItem (tempstyle, mainStyle);
  sprintf (styleStr, "%s.label:", style);
  strcpy (tempstyle, ReadString (path, styleStr, "ParentRelative"));
  if (editStyle)
    delete editStyle;
  editStyle = new StyleItem;
  ParseItem (tempstyle, editStyle);



  // ...and some additional parameters
  bevel = ReadInt (path, "bevelWidth:", 0);
  border = ReadInt (path, "borderWidth:", 0);

  if (ReadBool
      (extensionsrcPath (), "blackbox.appearance.bullet.unix:", true))
    height = fontHeight + 2 + (2 * bevel) + (border * 4);
  else
    height = fontHeight + 2 + (border * 2);
  if (borderWidth != -1)
    height = fontHeight + 2 + borderWidth * 2;
  if (bevelWidth != -1)
    height += bevelWidth * 2 + 2;

}


void
Broambox::SetWinFont (HWND win, HFONT font)
{
  if (font != NULL)
    DeleteObject (font);

  font = CreateFont (fontHeight, 0, 0, 0,
                     FW_DONTCARE, FALSE, FALSE, FALSE,
                     DEFAULT_CHARSET,
                     OUT_DEFAULT_PRECIS,
                     CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, 0, fontFace);

  SendMessage (win, WM_SETFONT, (WPARAM) font, FALSE);


}


void
Broambox::ReplaceFont ()
{
  if (hEditFont != NULL)
    DeleteObject (hEditFont);

  hEditFont = CreateFont (fontHeight, 0, 0, 0,
                          FW_DONTCARE, FALSE, FALSE, FALSE,
                          DEFAULT_CHARSET,
                          OUT_DEFAULT_PRECIS,
                          CLIP_DEFAULT_PRECIS,
                          ANTIALIASED_QUALITY, 0, fontFace);

  SendMessage (hEditWnd, WM_SETFONT, (WPARAM) hEditFont, FALSE);

  TEXTMETRIC TXM;
  HDC hdc = CreateCompatibleDC (NULL);
  SelectObject (hdc, hEditFont);
  GetTextMetrics (hdc, &TXM);
  DeleteDC (hdc);

  int realfontheight = TXM.tmHeight;    //pixel

  RECT r;
  GetClientRect (hEditWnd, &r);

  int extraspace = (r.bottom - realfontheight) / 2;
  if (extraspace < 0)
    extraspace = 0;

  r.left = extraspace + 1;
  r.right -= extraspace + 1;
  r.top = extraspace;
  r.bottom -= extraspace;

  SendMessage (hEditWnd, EM_SETRECT, 0, (LPARAM) & r);
}

void
Broambox::ExpandAlias (char *ret_val, char *alias_val, char *remaining,
                       char *args)
{
  char *arg;
  char tmp[MAX_LINE_LENGTH], atmp[MAX_LINE_LENGTH];
  strcpy (atmp, alias_val);

  arg = strstr (atmp, "<args>");
  if (arg)
    {
      *(arg) = '\0';
      arg += 6;
      strcpy (ret_val, atmp);
      strcat (ret_val, remaining);
      strcat (ret_val, arg);
    }
  else
    {
      strcpy (tmp, atmp);
      char *start = tmp;
      if (start && *start == '"')
        {
          start++;
          while (*start && *start != '"')
            start++;
        }
      char *remg = strchr (start, ' ');

      if (remg)
        {
          *(remg++) = '\0';
          strcpy (ret_val, tmp);
          strcpy (args, remg);
          if (remaining)
            strcat (args, remaining);
        }
      else
        {
          strcpy (ret_val, atmp);
          if (remaining)
            strcpy (args, remaining);
        }


    }
}
void
Broambox::ExpandAliases (char *buf, char *args)
{

  char *firstword;
  char *remaining;
  char *start;
  int found_match = 0;
  char tmp[MAX_LINE_LENGTH];
  strcpy (tmp, buf);
  start = tmp;
  if (start && *start == '"')
    {
      start++;
      while (*start && *start != '"')
        start++;
    }

  remaining = strchr (start, ' ');
  if (remaining)
    *(remaining++) = '\0';
  firstword = tmp;

  char ret_val[MAX_LINE_LENGTH];
  strcpy (ret_val, tmp);
  if (remaining)
    strcpy (args, remaining);
  else
    strcpy (args, "");

  if (firstword)
    {

      if (firstword[0] == '?')
        {
          int remln = strlen (remaining);
          /* Fix for search engines */
          for (int j = 0; j < remln; j++)
            {
              if (remaining[j] == ' ')
                remaining[j] = '+';
            }
        }

      char *expansion = AliasTrie->GetString (firstword);
      if (expansion)
        {
          ExpandAlias (ret_val, expansion, remaining, args);
          found_match = 1;

        }



      if (!found_match && firstword[0] == '?')
        {
          strcpy (ret_val, "http://www.google.com/search?q=");
          strcat (ret_val, remaining);
        }


    }

  strcpy (buf, ret_val);

}



void
Broambox::ExpandRegexs (char *buf, char **out)
{
  int i;
  int len, nlen;
  char *tbuf = buf;
  char *obuf;
  obuf = buf;



  for (i = 0; i < n_regex; i++)
    {
      len = strlen (tbuf);
      if (regexs[i])
        {

          pcrs_job *reg = regexs[i];

          int ret = pcrs_execute (reg, tbuf, len, &obuf, (size_t *) & nlen);
          reg = reg->next;

          while (reg != NULL && ret)
            {

              tbuf = strdup (obuf);
              len = strlen (tbuf);
              ret = pcrs_execute (reg, tbuf, len, &obuf, (size_t *) & nlen);
              reg = reg->next;
            }



        }
      tbuf = strdup (obuf);
    }

  *out = obuf;
}

void
Broambox::WriteAlias (char *str)
{
  char pathh[4096];
  sprintf (pathh, "%saliases.rc", localDir);

  FILE *f_ali = fopen (pathh, "a");
  if (!f_ali)
    f_ali = fopen (".\\aliases.rc", "w");
  if (f_ali)
    {
      fprintf (f_ali, "%s\n", str);
      fflush (f_ali);
      fclose (f_ali);
    }
}

void
Broambox::GetVarValue (char *varname, char *varvalue)
{
  char *str = VarTrie->GetString (varname);
  if (str)
    strcpy (varvalue, str);
  else
    strcpy (varvalue, "");

}

void
Broambox::ExpandVariable (char *str, int len)
{
  char tmp[MAX_LINE_LENGTH];
  char varname[MAX_LINE_LENGTH], varvalue[MAX_LINE_LENGTH];
  int i, invar = 0, k = 0, vn = 0;
  for (i = 0; i < len + 1; i++)
    {
      if (!invar && str[i] != '$')
        tmp[k++] = str[i];
      if (invar)
        {
          if (str[i] != ' ' && str[i] != '\0' && str[i] != '\t'
              && str[i] != '\n' && str[i]!='\\')
            varname[vn++] = str[i];
          else
            {
			  
              varname[vn] = '\0';

              GetVarValue (varname, varvalue);
              for (unsigned int j = 0; j < strlen (varvalue); j++)
                {
                  tmp[k++] = varvalue[j];
                }
				
			  if(str[i]!='\\' && str[i]!='\0')
				  tmp[k++] = ' ';
              invar = 0;
            }
        }
      else if (str[i] == '$')
        {
          vn = 0;
          invar = 1;
        }


    }
  tmp[k] = '\0';
  strcpy (str, tmp);
}



void
Broambox::ExpandVariables (char *buf, char *args)
{
	/* Expand recursively */
  int nexps = 0;
  while(strchr(buf, '$') || strchr(args, '$') && nexps<64)
  {
	ExpandVariable (buf, strlen (buf));
	ExpandVariable (args, strlen (args));
	nexps++;
  }
}


void
Broambox::Execute ()
{
  bool redirect = false;
  char *buf;
  char bufs[MAX_LINE_LENGTH], args[MAX_LINE_LENGTH];
  char hbuf[MAX_LINE_LENGTH];
  char *obuf;

  GetWindowText (hEditWnd, bufs, sizeof (bufs));
  buf = bufs;
  strcpy (hbuf, buf);


  if (*buf == '$')
    {
      if (ParseVariable (buf))
        {

          WriteAlias (buf);
          SetWindowText (hEditWnd, "");
          goto copy_to_history;
        }
    }
    else if (strlen (buf) > 2 && *buf == 's'
                   && *(buf+ 1) == '/'){
      FindRegexs (buf);
      WriteAlias(buf);
      goto copy_to_history;
    }
              
  if (*buf == '*')
    {

      redirect = true;
      buf++;
    }

  if (use_regexs)
    {

      ExpandRegexs (buf, &obuf);
      strcpy (buf, obuf);

    }

  ExpandAliases (buf, args);

  ExpandVariables (buf, args);

  if(regex_debug)
  {
    char msg[MAX_LINE_LENGTH]; 
	if(strlen(args)>0)
	{
		sprintf(msg, ">%s %s<", buf, args);
	}
	else
		sprintf(msg, ">%s<", buf);
    MessageBox (NULL, msg, NULL, MB_OK);
    return;
  }

  for (unsigned int i = 0; i < strlen (buf); i++)
    {
      if (isspace (buf[i]))
        buf[i] = ' ';
    }
  for (unsigned int i = 0; i < strlen (args); i++)
    {
      if (isspace (args[i]))
        args[i] = ' ';
    }

  if (strlen (buf) < 2)
    return;

  if (buf[0] == '@')
    {
	if(strlen(successSound)>1)
		  PlaySound(successSound, NULL, SND_FILENAME | SND_ASYNC);

      if(strlen(args)>0)
      {
        strcat (buf, " ");
        strcat (buf, args);
      }
      PostMessage (hBB4Win, BB_BROADCAST, 0, (LPARAM) strdup (buf));

    }
  else if (buf[0] == '[' && buf[strlen (buf) - 1] == ']')
    {
	  	if(strlen(successSound)>1)
		  PlaySound(successSound, NULL, SND_FILENAME | SND_ASYNC);

      if (!_stricmp (buf, "[exit]") || !_stricmp (buf, "[quit]"))
        PostMessage (hBB4Win, BB_QUIT, 0, 0);
      else if (!_stricmp (buf, "[shutdown]"))
        PostMessage (hBB4Win, BB_SHUTDOWN, 0, 0);
      else if (!_stricmp (buf, "[reboot]"))
        PostMessage (hBB4Win, BB_SHUTDOWN, 1, 0);
      else if (!_stricmp (buf, "[logoff]"))
        PostMessage (hBB4Win, BB_SHUTDOWN, 2, 0);
      else if (!_stricmp (buf, "[hibernate]"))
        PostMessage (hBB4Win, BB_SHUTDOWN, 3, 0);
      else if (!_stricmp (buf, "[suspend]"))
        PostMessage (hBB4Win, BB_SHUTDOWN, 4, 0);
      else if (!_stricmp (buf, "[lock]"))
        PostMessage (hBB4Win, BB_SHUTDOWN, 5, 0);
      else if (!_stricmp (buf, "[run]"))
        PostMessage (hBB4Win, BB_RUN, 0, 0);
      else if (!_stricmp (buf, "[reconfig]"))
        PostMessage (hBB4Win, BB_RECONFIGURE, 0, 0);
      else if (!_stricmp (buf, "[restart]"))
        PostMessage (hBB4Win, BB_RESTART, 0, 0);
      else if (!_stricmp (buf, "[editstyle]"))
        PostMessage (hBB4Win, BB_EDITFILE, 0, 0);
      else if (!_stricmp (buf, "[editmenu]"))
        PostMessage (hBB4Win, BB_EDITFILE, 1, 0);
      else if (!_stricmp (buf, "[editplugins]"))
        PostMessage (hBB4Win, BB_EDITFILE, 2, 0);
      else if (!_stricmp (buf, "[editextensions]"))
        PostMessage (hBB4Win, BB_EDITFILE, 3, 0);
      else if (!_strnicmp (buf, "[who", 4) && !noWho)
        {
          char temp[50];
          DWORD len = 50;
          GetUserName (temp, &len);
          temp[len] = '\0';

          SetWindowText (hEditWnd, temp);
          SetFocus (NULL);
          goto copy_to_history;
        }
    }
  else
    {
      if (redirect)
        {
          char tempredir[MAX_LINE_LENGTH];
          char tempfile[MAX_LINE_LENGTH];
          tmpnam (tempfile);
          sprintf (tempredir, "/c %s %s > %s", buf, args, tempfile);
          AddVariable ("tmpstar", tempfile);



          ShellExecute (NULL, NULL, "cmd.exe", tempredir, NULL, SW_HIDE);


          sprintf (tempredir, "@BBCore.edit %s", tempfile);
          Sleep (300);
          FILE *inf = fopen (tempfile, "r");
          if (inf)
            {
              char input[65536];
              input[fread (input, 1, sizeof (input), inf)] = '\0';

              AddVariable ("laststar", input);

            }
          else
            AddVariable ("laststar", "");
          PostMessage (hBB4Win, BB_BROADCAST, 0, (LPARAM) strdup (tempredir));

        }
      else
        {
         int success = (int) ShellExecute (NULL, NULL, buf, args, NULL, SW_SHOWNORMAL);
	if(success<=32)
		{int ovec[30];
	if(strlen(failSound)>1)
		  PlaySound(failSound, NULL, SND_FILENAME | SND_ASYNC);
		  

if(pcre_exec(urlre, urlre_extra, buf, strlen(buf), 0, 0, ovec, 30)>0)
{
char tmpurl[MAX_LINE_LENGTH];
sprintf(tmpurl, "http://%s", buf);
ShellExecute(NULL, NULL, tmpurl, "", NULL, SW_SHOWNORMAL);
}		

		}
	else
	{
		if(strlen(successSound)>1)
		  PlaySound(successSound, NULL, SND_FILENAME | SND_ASYNC);

	}
        }
    }

  AddVariable ("lastcmd", buf);
  AddVariable ("lastargs", args);
  SetWindowText (hEditWnd, "");

copy_to_history:
  if(buf[0]!='@' && args)
  {	
	strcat(buf, " ");
	strcat(buf, args);
  }

  PostMessage(hBB4Win, BB_SETTOOLBARLABEL, NULL, (LPARAM) buf);
  AddVariable ("lastunex", hbuf);


  if (strlen (hbuf) > 1)
    {

      int nLen;
      char path[MAX_LINE_LENGTH], pathh[MAX_LINE_LENGTH];
      GetModuleFileName (hMainInst, path, sizeof (path));
      nLen = strlen (path) - 1;
      while (nLen > 0 && path[nLen] != '\\')
        nLen--;
      path[nLen + 1] = 0;

      sprintf (pathh, "%shistory.rc", path);

      for (int i = historyCount - 1; i >= 0; i--)
        strcpy (history[i + 1], history[i]);

      strcpy (history[0], hbuf);

      if (historyCount < HIST_SIZE)
        historyCount++;

      FILE *f_hist = fopen (pathh, "a");

      if (!f_hist)
        f_hist = fopen (".\\history.rc", "w");
      if (f_hist)
        {

          fprintf (f_hist, "%s\n", hbuf);
          fflush (f_hist);
          fclose (f_hist);
        }

    }



}

void
Broambox::ShowWindow ()
{
  shown = true;
  if (!InSlit ())
    ShowWindowAt (x, y);
}

void
Broambox::ShowWindowAt (int x, int y)
{
  shown = true;
  if (InSlit ())
    {
      tempSlit = true;
      LeaveSlit ();
    }
  SetWindowPos (hEditWnd, HWND_TOP, x, y, 0, 0,
                SWP_NOSENDCHANGING | SWP_NOSIZE | SWP_SHOWWINDOW);
}

void
Broambox::HideWindow ()
{
  shown = false;
  if (tempSlit && hSlit != NULL)
    {
      EnterSlit ();
      tempSlit = false;
    }
  else if (!InSlit ())
    SetWindowPos (hEditWnd, NULL, x, y, 0, 0,
                  SWP_NOZORDER | SWP_NOSENDCHANGING | SWP_NOSIZE |
                  SWP_HIDEWINDOW);
}

void
Broambox::EnterSlit ()
{
  if (!InSlit ())
    {
      SendMessage (hSlit, SLIT_ADD, NULL, (LPARAM) hEditWnd);
      //SizeBroambox();
      inSlit = true;
    }
}

void
Broambox::LeaveSlit ()
{
  if (InSlit ())
    {
      SendMessage (hSlit, SLIT_REMOVE, NULL, (LPARAM) hEditWnd);
      //SizeBroambox();
      inSlit = false;
    }
}

void
Broambox::UpdateSlit ()
{
  if (InSlit ())
    SendMessage (hSlit, SLIT_UPDATE, NULL, NULL);
}

void
Broambox::ToggleSlitAutoHide (bool getSetting)
{
  getSetting; //Avoid warning
  return;
/*
  if (InSlit ())
    {
      if (getSetting)
	isSlitAutoHide =
	  ReadBool (bbrcPath (), "session.screen0.slit.autohide:", false);

      if (isSlitAutoHide && getSetting)
	SendMessage (hBB4Win, BB_BROADCAST, NULL,
		     (LPARAM) "@BBSlit Toggle Hide Off");
      else if (isSlitAutoHide)
	{
	  SendMessage (hBB4Win, BB_BROADCAST, NULL,
		       (LPARAM) "@BBSlit Toggle Hide On");
	  isSlitAutoHide = false;
	}
    }
	*/
}



void
Broambox::AutoCompletePath (char *path)
{
  int n, k;
  char tmp[MAX_LINE_LENGTH];
  bool quoted = false;
  int qindex = 0;

  strcpy (tmp, path);
  n = strlen (tmp) - 1;
  k = strlen (tmp) - 1;
  while (tmp[k] != '\\' && k > 0)
    k--;
  k++;
  while (tmp[n] == ' ' && n > 0)
    n--;

  if (tmp[n] == '"')
    {
      quoted = true;
      qindex = n;
      n--;
      while (tmp[n] != '"' && n > 0)
        n--;
    }
  else
    while (tmp[n] != ' ' && n > 0)
      n--;
  n++;
  if (!quoted)
    strcat (tmp, "*");
  else
    {
      tmp[qindex] = '*';
      strcat (tmp, "\"");
    }


  WIN32_FIND_DATA fd;
  if (!LastACHandle)
    LastACHandle = FindFirstFile (tmp + n, &fd);
  else
    FindNextFile (LastACHandle, &fd);
  if(LastACHandle!=INVALID_HANDLE_VALUE && !strcmp(fd.cFileName, "."))
	  FindNextFile (LastACHandle, &fd);
  if(LastACHandle!=INVALID_HANDLE_VALUE && !strcmp(fd.cFileName, ".."))

  if (LastACHandle != INVALID_HANDLE_VALUE && strlen (fd.cFileName) < 2)
    LastACHandle = FindFirstFile (tmp + n, &fd);
  if (LastACHandle != INVALID_HANDLE_VALUE)

    {



	  if(!quoted && strchr(fd.cFileName, ' '))
	  {
		char qtmp[MAX_LINE_LENGTH];
		strcpy(qtmp, tmp);
		qtmp[n] = '\"';
		strcpy(qtmp+n+1, tmp+n);
		quoted = 1;
		k++;
		strcpy(tmp, qtmp);

	  }

      strcpy (tmp + k, fd.cFileName);
	  if(quoted)
		  strcat(tmp, "\"");
      SetWindowText (hEditWnd, tmp);
	  if(quoted)
		  SendMessage (hEditWnd, EM_SETSEL, strlen (tmp)-1, strlen (tmp)-1);
	  else
          SendMessage (hEditWnd, EM_SETSEL, strlen (tmp), strlen (tmp));

    }
  else
    {
      LastACHandle = NULL;

    }

}


void
Broambox::AutoCompleteReset (void)
{
  lastTabCompleteIndex = 0;
}

void
Broambox::AutoCompleteNext (char *cur, int istab)
{
  int j;
  int prev = lastTabCompleteIndex - 1;
  if (prev < 0)
    prev += historyCount;
  for (int i = 0; i < historyCount; i++)
    {
      j = (i + lastTabCompleteIndex) % historyCount;

      if (strstr (history[j], cur) == history[j]
          && (!istab || _stricmp (history[j], history[prev])))
        {

          SetWindowText (hEditWnd, history[j]);
          lastTabCompleteIndex = j + 1;
          break;
        }
    }
  SendMessage (hEditWnd, EM_SETSEL, (WPARAM)0, (LPARAM)-1);
  SendMessage (hEditWnd, EM_SETSEL, (WPARAM)-1, (LPARAM)0);
}

void
Broambox::AutoCompletePrev (char *cur)
{
  int j, k;
  for (int i = 0; i < historyCount; i++)
    {
      k = (i + lastTabCompleteIndex) % historyCount;
      j = historyCount - k;


      if (strstr (history[j], cur) == history[j])
        {
          SetWindowText (hEditWnd, history[j]);
          lastTabCompleteIndex = j - 1;
          if (lastTabCompleteIndex < 0)
            lastTabCompleteIndex += historyCount;
          break;
        }
    }
  SendMessage (hEditWnd, EM_SETSEL, 0, 0xffffffff);
  SendMessage (hEditWnd, EM_SETSEL, 0xffffffff, 0);
}


void
Broambox::AutoComplete (char *cur)
{
  for (int i = 0; i < historyCount; i++)
    {
      if (strstr (history[i], cur) == history[i])
        SetWindowText (hEditWnd, history[i]);


    }

  SendMessage (hEditWnd, EM_SETSEL, 0, 0xffffff);
  SendMessage (hEditWnd, EM_SETSEL, 0xffffff, 0);
}


void
Broambox::HistoryNext ()
{
  char *szBuf;

  if (historyIndex >= historyCount)
    {
      szBuf = history[historyCount - 1];
      historyIndex = historyCount;
    }
  else
    szBuf = history[historyIndex++];

  SetWindowText (hEditWnd, szBuf);

  SendMessage (hEditWnd, EM_SETSEL, 0, 0xffffffff);
  SendMessage (hEditWnd, EM_SETSEL, 0xffffffff, 0);
}

void
Broambox::HistoryPrev ()
{
  char *szBuf;

  if (historyIndex <= 0)
    {
      szBuf = "";
      historyIndex = 0;
    }
  else
    {
      if (historyIndex != 1)
        szBuf = history[historyIndex - 2];
      else
        szBuf = "";
      historyIndex--;
    }

  SetWindowText (hEditWnd, szBuf);

  SendMessage (hEditWnd, EM_SETSEL, 0, 0xffffffff);
  SendMessage (hEditWnd, EM_SETSEL, 0xffffffff, 0);
}

void Broambox::ShowAliases()
{
 ShowTrie(AliasTrie);
}

void Broambox::ShowVariables()
{
  ShowTrie(VarTrie);
}

void Broambox::ShowRegexs()
{
  char out[65536];
  char line[MAX_LINE_LENGTH];
  if(RegexList!=NULL)	  {
	StrList *strlist = RegexList->head;
  out[0] = 0;

  
  while(strlist!=NULL)
  {
	if(strlist->str && strlen(strlist->str)>1)
	{sprintf(line, "%s\r\n", strlist->str);
	strcat(out, line);
	}
	strlist = strlist->next;

  }
 ShowString(out);
  }
  else
  {
	ShowString("");

  }
  
}

void Broambox::ShowTrie(trie *tr)
{
  char out[65536];
  out[0] = '\0';
 tr->ReturnJoinedString (out, "");
 ShowString(out);
}

void Broambox::ShowString(char *out)
{
      RECT rect;
      RECT drect;
      GetWindowRect (GetDesktopWindow (), &drect);
      GetWindowRect (hEditWnd, &rect);
      int bx = rect.left + 800;
      int by = rect.top + 500;
      int dx = 0, dy = 0;
      if (bx > drect.right)
        dx = drect.right - bx;
      if (by > drect.bottom)
        dy = drect.bottom - by;


      HWND hAliasWnd = CreateWindowEx (WS_EX_TOOLWINDOW,
                                       "EDIT",
                                       "Aliases",
                                       WS_POPUP | WS_VISIBLE | WS_VSCROLL
                                       | ES_MULTILINE | WS_OVERLAPPEDWINDOW,
                                       rect.left + dx, rect.top + dy, 800,
                                       500,
                                       NULL,
                                       NULL,
                                       hMainInst,
                                       NULL);


      SetWindowText (hAliasWnd, out);
      SetWinFont (hAliasWnd, aliasFont);
}


LRESULT
  Broambox::RButtonDown (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

if (GetAsyncKeyState (VK_CONTROL) & 0x8000)
{

	Menu *uberMenu = MakeNamedMenu("UberBox", "UberBox", true);
	MakeMenuItem(uberMenu, "Show Aliases", "@BBBroamBoxShowAliases", false);
	MakeMenuItem(uberMenu, "Show Variables", "@BBBroamBoxShowVariables", false);
	MakeMenuItem(uberMenu,  "Show Regexs", "@BBBroamBoxShowRegexs", false);
	MakeMenuNOP(uberMenu,NULL);
    Menu *settingsMenu = MakeNamedMenu("Settings", "UberBoxSettings", false);
	MakeSubmenu(uberMenu, settingsMenu, "Settings");
    MakeMenuItem(settingsMenu, "Reload Settings", "@BBBroamBoxReloadSettings",false);
	MakeMenuItem(settingsMenu, "Edit Settings", "@BBBroamBoxEditSettings",false);
	MakeMenuItem(settingsMenu, "Save Settings", "@BBBroamBoxSaveSettings",false);
    Menu *styleMenu = MakeNamedMenu("Style", "UberBoxStyle", false);
	MakeSubmenu(uberMenu, styleMenu, "Style");
	MakeMenuItem(styleMenu, "Toggle Autohide", "@BBBroamBoxAutohide",false);
	MakeMenuItem(styleMenu, "Toggle Autohide Always", "@BBBroamBoxAutohideAlways",false);
	MakeMenuItem(styleMenu, "Toggle In Slit", "@BBBroamBoxInSlit", false);	
	MakeMenuItemInt(styleMenu, "Width", "@BBBroamBoxWidth", width, 1, 2000);
	MakeMenuItemInt(styleMenu, "Bevel Width", "@BBBroamBoxBevelWidth", bevelWidth, 0, 50);	
	MakeMenuNOP(styleMenu, NULL);
    MakeMenuNOP(styleMenu, "Style");
	bool styleCheck;
	
	styleCheck = !_stricmp(configStyle, "toolbar");
    MakeMenuItem(styleMenu, "toolbar", "@BBBroamBoxStyle toolbar", styleCheck);
	styleCheck = !_stricmp(configStyle, "toolbar.label");
	MakeMenuItem(styleMenu, "toolbar.label", "@BBBroamBoxStyle toolbar.label", styleCheck);
	styleCheck = !_stricmp(configStyle, "toolbar.clock");
	MakeMenuItem(styleMenu, "toolbar.clock", "@BBBroamBoxStyle toolbar.clock", styleCheck);
	styleCheck = !_stricmp(configStyle, "toolbar.button");
	MakeMenuItem(styleMenu, "toolbar.button", "@BBBroamBoxStyle toolbar.button", styleCheck);
	styleCheck = !_stricmp(configStyle, "toolbar.button.pressed");
	MakeMenuItem(styleMenu, "toolbar.button.pressed", "@BBBroamBoxStyle toolbar.button.pressed", styleCheck);
	styleCheck = !_stricmp(configStyle, "toolbar.windowLabel");
	MakeMenuItem(styleMenu, "toolbar.windowLabel", "@BBBroamBoxStyle toolbar.windowLabel", styleCheck);


	
	MakeMenuNOP(uberMenu, NULL);
	MakeMenuItem(uberMenu, "Toggle Use Regexs", "@BBBroamBoxUseRegexs", false);
	MakeMenuItem(uberMenu, "Toggle Debug mode", "@BBBroamBoxRegexDebug", false);
	
	MakeMenuItem(uberMenu, "Last Redirected File", "@BBBroamBoxLastStar", false);
	

	ShowMenu(uberMenu);

     return 0;
}
else 
     
  return CallWindowProc (wpEditProc, hWnd, uMsg, wParam, lParam);;
}
